Require Import Coqlib.
Require Import Maps.
Require Import VTDefs.
Require Import Integers.
Require Import Values.
Require Import LemmaVTMap.
Require Import LemmaInterference.
Require Import LemmaOtherWorkload.
Require Import LemmaOtherWorkload1.
Require Import LemmaSchedulableWorkload.
Require Import LemmaMaths.
Require Import Psatz.
Require Import LemmaWorkloadEnlargeSelf.
Require Import LemmaWorkloadOnOthers.

Local Open Scope Z_scope.

Section ENLARGE.

  Theorem enlarge_preserves_schedulability:
    forall N conf conf' id k T C,
      1 < k ->
      0 <= id < N ->
      ZMap.get id conf = mkPrioConfigValid T C ->
      config_pool_valid N conf ->
      config_pool_valid N conf' ->
      id_is_enlarged_by_k id k conf conf' ->
      enlarged_task_has_largest_period id k N conf ->
      system_schedulable N conf ->
      system_schedulable N conf'.
  Proof.
    intros * Hk Hid Hconfid Hconfvalid Hconfvalid' Henlarge Hlarge Hsched.
    apply workload_schedulable_implies_system_schedulable; auto.

    unfold workload_schedulable.
    intros * Hi Hconfi' p Hp.
    pose proof Henlarge as Henlarge0.
    unfold id_is_enlarged_by_k in Henlarge.
    specialize (Henlarge i T C Hconfid).
    
    destruct (zeq i id) as [Hself|Hother].
    {
      rewrite Hself in *. clear Hself.
      rewrite ZMap.gss in Henlarge.
      rewrite Hconfi' in Henlarge.
      inv Henlarge.

      transitivity (workload_total1 (Z.to_nat N) id (p * (k * T)) (p * (k * T) + k * T) conf
                                    (TAC_pool_at_t (p * (k * T) + k * T) (Z.to_nat N) conf)).
      {
        replace (p * (k * T)) with (p * k * T) by lia.
        apply enlarge_does_not_incur_more_workload_on_self with (C:=C); auto.
      }
      {
        apply workloads_bounded_if_schedulable_self; auto.
        - intros * Hi0 Hconfi0.
          unfold enlarged_task_has_largest_period in Hlarge.
          specialize (Hlarge _ _ _ _ _ Hi0 Hconfid Hconfi0).
          auto.
      }
    }
    {
      rewrite ZMap.gso in Henlarge; auto.
      rewrite Henlarge in Hconfi'.      
      transitivity (workload_total1 (Z.to_nat N) i (p * T0) (p * T0 + T0) conf
                                    (TAC_pool_at_t (p * T0 + T0) (Z.to_nat N) conf)).
      {
        apply enlarge_does_not_incur_more_workload_on_others with (id:=id) (k:=k) (C:=C0); auto.
      }
      {
        apply workloads_bounded_if_schedulable_others; auto.
      }
    }
  Qed.
        
End ENLARGE.


