Require Import Coqlib.
Require Import Maps.
Require Import VTDefs.
Require Import Integers.
Require Import Values.
Require Import LemmaVTMap.
Require Import LemmaInterference.
Require Import LemmaOtherWorkload.
Require Import LemmaOtherWorkload1.
Require Import LemmaSchedulableWorkload.
Require Import LemmaMaths.
Require Import Psatz.


Local Open Scope Z_scope.

Section SUMBUDGET.

  Fixpoint sum_budget (N: nat) (conf: PrioConfigPool) :=
    match N with
    | O => 0
    | S n => match (ZMap.get (Z.of_nat n) conf) with
            | mkPrioConfigValid T C =>
              C + (sum_budget n conf)
            end
    end.

End SUMBUDGET.
  

Section WORKLOAD_REMAIN.

  Lemma workload_remain_0_bound_tasks:
    forall H N conf i t TAC,
      0 <= N ->
      (forall i T C,
          0 <= i < N ->
          ZMap.get i conf = mkPrioConfigValid T C ->
          T = H) ->
      t mod H = 0 ->
      workload_remain_total (Z.to_nat N) i t (t + H) conf TAC = 0.
  Proof.
    intros * HN Hsame Hmod.
    remember (Z.to_nat N) as Nnat.
    assert (HNN: N = Z.of_nat Nnat).
    {
      rewrite HeqNnat.
      rewrite Z2Nat.id; omega.
    }
    rewrite HNN in Hsame.
    clear HNN HeqNnat.

    induction Nnat.
    {
      simpl.
      auto.
    }
    {
      assert (IH1: forall i0 T C : Z,
                 0 <= i0 < Z.of_nat Nnat -> ZMap.get i0 conf = mkPrioConfigValid T C -> T = H).
      {
        intros * Hi Hconfi.
        assert (Hi': 0 <= i0 < Z.of_nat (S Nnat)). xomega.
        specialize (Hsame _ _ _ Hi' Hconfi).
        auto.
      }
      specialize (IHNnat IH1).
      
      simpl.
      rewrite IHNnat.
      destruct (zeq i (Z.of_nat Nnat)); auto.
      destruct (ZMap.get (Z.of_nat Nnat) conf) as [T C] eqn:Hconfn.
      assert (Hn: 0 <= Z.of_nat Nnat < Z.of_nat (S Nnat)). xomega.
      specialize (Hsame _ _ _ Hn Hconfn).
      rewrite Hsame in *.
      unfold workload_remain.
      rewrite Hmod.
      rewrite zeq_true; auto.
    }
  Qed.

End WORKLOAD_REMAIN.


Section WORKLOAD_WHOLE.

  Lemma workload_whole_bound_tasks_aux1:
    forall H N conf i t,
      0 <= t ->
      0 <= N <= i ->
      config_pool_valid N conf ->
      (forall i T C,
          0 <= i < N ->
          ZMap.get i conf = mkPrioConfigValid T C ->
          T = H) ->
      t mod H = 0 ->
      workload_whole_total (Z.to_nat N) i t (t + H) conf <= sum_budget (Z.to_nat N) conf.
  Proof.
    intros * Ht Hi Hconf Hsame Hmod.
    remember (Z.to_nat N) as Nnat.
    assert (HNN: N = Z.of_nat Nnat).
    {
      rewrite HeqNnat.
      rewrite Z2Nat.id; omega.
    }
    rewrite HNN in *.
    clear HNN HeqNnat.
    
    induction Nnat.
    {
      simpl.
      omega.
    }
    {
      simpl.
      rewrite zeq_false; try xomega.
      destruct (ZMap.get (Z.of_nat Nnat) conf) as [T C] eqn:Hconfn.
      assert (Hn: 0 <= Z.of_nat Nnat < Z.of_nat (S Nnat)). xomega.
      pose proof (Hsame _ _ _ Hn Hconfn) as HT.
      rewrite HT in *. clear HT.
      pose proof (Hconf _ _ _ Hn Hconfn) as HTC.

      assert (H1: (t + H) mod H = 0).
      {
        rewrite mod_0_preserve; try omega.
      }
      
      assert (Hle1: workload_whole i (Z.of_nat Nnat) H C t (t + H) <= C).
      {
        unfold workload_whole.
        rewrite zlt_false; try omega.
        rewrite H1.
        rewrite zeq_true.
        repeat rewrite mod_0_divide_mul; try omega.
        rewrite Hmod.
        rewrite zeq_true.
        
        destruct (zlt (Z.of_nat Nnat) i).
        - replace (t + H - t) with H by omega.
          rewrite Z_div_same; try omega.
        - replace (t + H - H - t) with 0 by omega.
          omega.
      }

      assert (HS: forall a b c d, a <= c -> b <= d -> a + b <= c + d). intros. omega.
      apply HS; auto.

      apply IHNnat; auto; try omega.
      - apply config_pool_valid_shrink with (N2:= Z.of_nat (S Nnat)); auto; try omega.
      - intros * Hx Hconfx.
        assert (Hx': 0 <= i0 < Z.of_nat (S Nnat)). omega.
        specialize (Hsame _ _ _ Hx' Hconfx).
        auto.
    }
  Qed.
  
  Lemma workload_whole_bound_tasks_aux2:
    forall H C N conf i t,
      0 <= t ->
      0 <= i < N ->
      ZMap.get i conf = mkPrioConfigValid H C ->
      config_pool_valid N conf ->
      (forall i T C,
          0 <= i < N ->
          ZMap.get i conf = mkPrioConfigValid T C ->
          T = H) ->
      t mod H = 0 ->
      workload_whole_total (Z.to_nat N) i t (t + H) conf <= sum_budget (Z.to_nat N) conf - C.
  Proof.
    intros * Ht Hi Hconfi Hconf Hsame Hmod.
    remember (Z.to_nat N) as Nnat.
    assert (HNN: N = Z.of_nat Nnat).
    {
      rewrite HeqNnat.
      rewrite Z2Nat.id; omega.
    }
    rewrite HNN in *.
    clear HNN HeqNnat.
    
    induction Nnat.
    {
      simpl in *.
      omega.
    }
    {
      simpl.
      assert (IH1: config_pool_valid (Z.of_nat Nnat) conf).
      {
        eapply config_pool_valid_shrink; eauto.
        xomega.
      }
      assert (IH2: forall i0 T C0 : Z, 0 <= i0 < Z.of_nat Nnat -> ZMap.get i0 conf = mkPrioConfigValid T C0 -> T = H).
      {
        intros * Hx Hconfx.
        assert (Hx': 0 <= i0 < Z.of_nat (S Nnat)). xomega.
        specialize (Hsame _ _ _ Hx' Hconfx).
        auto.
      }
      
      destruct (zeq i (Z.of_nat Nnat)) as [Hskip|Hcount].
      {
        rewrite Hskip in *. clear Hskip.
        rewrite Hconfi.
        replace (C + sum_budget Nnat conf - C) with (sum_budget Nnat conf) by omega.
        assert (HNN: Nnat = Z.to_nat (Z.of_nat Nnat)).
        {
          rewrite Nat2Z.id; auto.
        }
        rewrite HNN.
        rewrite Z2Nat.id; try omega.
        apply workload_whole_bound_tasks_aux1; auto.
        - omega.
      }
      {
        destruct (ZMap.get (Z.of_nat Nnat) conf) as [T' C'] eqn:Hconfn.
        assert (Hn: 0 <= Z.of_nat Nnat < Z.of_nat (S Nnat)). xomega.
        pose proof (Hsame _ _ _ Hn Hconfn) as HT'.
        rewrite HT' in *. clear HT'.
        pose proof (Hconf _ _ _ Hn Hconfn) as HTC'.

        assert (H1: (t + H) mod H = 0).
        {
          rewrite mod_0_preserve; try omega.
        }
        
        assert (Hle: workload_whole i (Z.of_nat Nnat) H C' t (t + H) <= C').
        {
          unfold workload_whole.
          rewrite zlt_false; try omega.
          rewrite H1.
          rewrite zeq_true.
          repeat rewrite mod_0_divide_mul; try omega.
          rewrite Hmod.
          rewrite zeq_true.
          
          destruct (zlt (Z.of_nat Nnat) i).
          - replace (t + H - t) with H by omega.
            rewrite Z_div_same; try omega.
          - replace (t + H - H - t) with 0 by omega.
            rewrite Zdiv_0_l.
            omega.
        }

        replace (C' + sum_budget Nnat conf - C) with (C' + (sum_budget Nnat conf - C)) by omega.
        assert (HS: forall a b c d, a <= c -> b <= d -> a + b <= c + d). intros. omega.
        apply HS; auto.

        apply IHNnat; auto.
        xomega.
      }
    }
  Qed.
  
  Lemma workload_whole_bound_tasks:
    forall H C N conf i t,
      0 <= t ->
      config_pool_valid N conf ->
      0 <= i < N ->
      ZMap.get i conf = mkPrioConfigValid H C ->
      (forall i T C,
          0 <= i < N ->
          ZMap.get i conf = mkPrioConfigValid T C ->
          T = H) ->
      t mod H = 0 ->
      sum_budget (Z.to_nat N) conf <= H ->      
      workload_whole_total (Z.to_nat N) i t (t + H) conf <= H - C.
  Proof.    
    intros * Ht Hconf Hi Hconfi Hsame Hmod Hsum.
    transitivity (sum_budget (Z.to_nat N) conf - C); try omega.
    apply workload_whole_bound_tasks_aux2; auto.
  Qed.

End WORKLOAD_WHOLE.

Section MAIN.

  Theorem same_period_schedulable:
    forall H N conf,
      (forall i T C,
          0 <= i < N ->
          ZMap.get i conf = mkPrioConfigValid T C ->
          T = H) ->
      config_pool_valid N conf ->
      sum_budget (Z.to_nat N) conf <= H ->
      system_schedulable N conf.
  Proof.
    intros * Hsame Hconf Hsum.
    apply workload_schedulable_implies_system_schedulable; auto.
    unfold workload_schedulable.
    intros * Hi Hconfi p Hp.
    unfold workload_total1.

    pose proof (Hsame _ _ _ Hi Hconfi) as HT.
    rewrite HT in *. clear HT.
    assert (H1: (p * H) mod H = 0).
    {
      apply Z_mod_mult.
    }
    rewrite workload_remain_0_bound_tasks; auto; try omega.
    rewrite Z.add_0_l.

    apply workload_whole_bound_tasks; auto.
    - specialize (Hconf _ _ _ Hi Hconfi).
      nia.
  Qed.
    
End MAIN.

