Require Import Coqlib.
Require Import Maps.
Require Import VTDefs.
Require Import Integers.
Require Import Values.
Require Import LemmaVTMap.
Require Import LemmaInterference.
Require Import LemmaOtherWorkload.
Require Import LemmaOtherWorkload2.
Require Import LemmaSchedulableWorkload.
Require Import LemmaWorkloadEnlargeSelf.
Require Import LemmaMaths.
Require Import Coq.Sorting.Mergesort.
Require Import Coq.Structures.Orders.
Require Import Coq.Sorting.Permutation.
Require Import Psatz.


Local Open Scope Z_scope.

Section MAIN.

  Theorem enlarge_does_not_incur_more_workload_on_others:
    forall N conf conf2 id k,
      1 < k ->
      0 <= id < N ->
      config_pool_valid N conf ->
      config_pool_valid N conf2 ->
      id_is_enlarged_by_k id k conf conf2 ->
      enlarged_task_has_largest_period id k N conf ->
      system_schedulable N conf ->
      (forall p i T C , 0 <= p ->
                   0 <= i < N ->
                   i <> id ->
                   ZMap.get i conf = mkPrioConfigValid T C ->                   
          workload_total1 (Z.to_nat N) i (p * T) (p * T + T) conf2
                            (TAC_pool_at_t (p * T + T) (Z.to_nat N) conf2) <=
            workload_total1 (Z.to_nat N) i (p * T) (p * T + T) conf
                            (TAC_pool_at_t (p * T + T) (Z.to_nat N) conf)).
  Proof.
    intros * Hk Hid Hconfvalid Hconf2valid Henlarged Hperiod Hschedulable * Hp Hi Hid' Hconfi.
    unfold workload_total1.

    remember (start_of_burst N conf2 (p*T) i (p*T + T)) as v.

    assert (H1: (p*T) mod T = 0).
    {
      apply Z_mod_mult.
    }
    assert (H2: ZMap.get i conf2 = mkPrioConfigValid T C).
    {
      destruct (ZMap.get id conf) as [Tid Cid] eqn:Hconfid.
      rewrite (Henlarged _ _ _ Hconfid).
      rewrite ZMap.gso; auto.
    }
    assert (H5: 1 < T).
    { specialize (Hconfvalid _ _ _ Hi Hconfi). omega. }    
    assert (H4: 0 <= v <= p*T).
    {
      rewrite Heqv.
      apply start_of_burst_result_range; auto.
      nia.
    }
    
    assert (Hbusy: workload_overlapping_total (Z.to_nat N) i v (p*T) (p*T+T) conf2
                - workload_remain_total (Z.to_nat N) i (p*T) (p*T+T) conf2 (TAC_pool_at_t (p * T + T) (Z.to_nat N) conf2) >= p*T - v - workload_extended_whole_total (Z.to_nat N) conf2 v (p*T)).
    {
      apply workloads_in_a_busy_schedule_one_burst with (C:=C); auto.
      nia.
    }

    assert (Hschedulable': workload_extended_whole_total (Z.to_nat N) conf v (p*T) +  workload_overlapping_total (Z.to_nat N) i v (p*T) (p*T+T) conf
                                                    - workload_remain_total (Z.to_nat N) i (p*T) (p*T+T) conf (TAC_pool_at_t (p * T + T) (Z.to_nat N) conf) <= p*T - v).
    {
      apply all_components_fit_in_one_burst with (C:=C); auto; try omega.
    }
    
    assert (Hcompare: workload_overlapping_total (Z.to_nat N) i v (p*T) (p*T+T) conf2 + workload_extended_whole_total (Z.to_nat N) conf2 v (p*T) + workload_whole_total (Z.to_nat N) i (p * T) (p * T + T) conf2 <=
                workload_overlapping_total (Z.to_nat N) i v (p*T) (p*T+T) conf + workload_extended_whole_total (Z.to_nat N) conf v (p*T) + workload_whole_total (Z.to_nat N) i (p * T) (p * T + T) conf).
    {
      apply compare_workloads_when_a_task_is_enlarged with (C:=C) (id:=id) (k:=k); auto; try omega.
    }
    omega.
  Qed.
      
End MAIN.



