(* This is a supplement to AuxStatedatatype.v
   TODO: merge later *)

Require Import Coqlib.
Require Import Maps.
Require Import Values.
Require Import Integers.
Require Import ZArith.
Require Import LemmaMaths.
Require Import Psatz.

Open Scope Z_scope.

Section PriorityScheduling.

  Inductive prio_config: Type :=
    | mkPrioConfigValid: Z (* period *) -> Z (* WCET *) -> prio_config.

  Definition PrioConfigPool: Type := ZMap.t prio_config.
  
  Definition is_a_hyperperiod (H: Z) (N: Z) (configs: PrioConfigPool) :=
    1 < H /\
    forall i T C,
      0 <= i < N ->
      ZMap.get i configs = mkPrioConfigValid T C ->
      H mod T = 0.

  Definition config_pool_valid (N: Z) (configs: PrioConfigPool) :=
    forall i T C,
      0 <= i < N ->
      ZMap.get i configs = mkPrioConfigValid T C ->
      (1 < T /\ T < Int.max_unsigned /\ 0 < C /\ C < T).

  Lemma config_pool_valid_shrink:
    forall (N1 N2: Z) conf,
      N1 <= N2 ->
      config_pool_valid N2 conf ->
      config_pool_valid N1 conf.
  Proof.
    intros * H1 H2.
    unfold config_pool_valid in *.
    intros * Hi Hconf.
    eapply H2; eauto; try omega.
  Qed.

  (* quantum value is indexed by priority level, from 0 up to num_prio_proc - 1 *)
  Definition QuantumPool: Type := ZMap.t Z.  

End PriorityScheduling.

Section TimeManagement.

  Definition vtmap: Type := Z -> Z.

  (* Starting from priority 0 *)
  Definition TimeContainerPool: Type := ZMap.t vtmap.

  Definition real_time_container_pool : TimeContainerPool :=
    ZMap.init (fun x => 0).  

End TimeManagement.



